/*
 * showprocbd: RIB filter for displaying the bounding box of a Procedural primitive
 *
 * PARAMETERS:
 *
 * boxalpha - single float alpha value used to set box side opacity
 * boxcolor  - optional r g b box color; overrides the current color attribute
 *
 * linewidth - width of lines representing box edges; must be set for lines to appear
 * linealpha - single float alpha value used to set line opacity
 * linecolor - optional r g b line color; overrides the current color attribute
 * linetype - set the curve type for line edges (polyline, ribbon, or tube)
 *
 * cornerwidth - width of spheres representing box corners
 * corneralpha - single float alpha value used to set corner opacity
 * cornercolor - optional r g b corner color; overrides the current color attribute
 *
 * surface - name of surface shader for box elements (default=defaultsurface)
 *
 * replace - when set to 0, procedural geometry is rendered along with the box
 *
 * HISTORY:
 *   Nov 1, 2010:  modified to omit Massive options proc
 *   July 21, 2010:  created by Scott Iverson
 *
 * COPYRIGHT 2010 SiTex Graphics, Inc.
 */

#include <stdio.h>
#include <stdlib.h>

#define BUFSIZE 1000000
#define MAXTOKENS 256

int Tokenize(char *s, int maxn, char **t) {
   int i=0, n=0;
   int b,q;

   while (1) {
      while ((s[i]<=' ') && (s[i]>'\00')) i++;
      if (s[i]=='\0') return n;
      b=(s[i]=='[');
      if (b) {
         i++; *t=&(s[i]); t++;
         while ((s[i]!=']') && (s[i]>'\0')) i++;
         n++;
         if (s[i]=='\0') return n;
         s[i]='\0'; i++;
      } else {
         q=(s[i]=='\"');
         if (q) {
            i++; *t=&(s[i]); t++;
            while ((s[i]!='\"') && (s[i]!='\0')) i++;
            n++;
            if (s[i]=='\"') {
               s[i]='\0'; i++; }
            else
               return n;
         } else {
            *t=&(s[i]); t++;
            while (s[i]>' ') i++;
            n++;
            if (s[i]=='\0') return n;
            s[i]='\0'; i++;
         };
      };
      if (n>=maxn) return n;
   }
}

int streq(char *a, char *b)
{
  return (strcmp(a,b)==0);
}

int main(int argc, char **argv)
{
  char buf[BUFSIZE];
  int i,ntk;
  char *tk[MAXTOKENS];
  char *bdp;
  char surfname[99], curvetype[99];

  double boxalpha = 1.0;
  double boxR=-1.0, boxG=-1.0, boxB=-1.0;

  double linewidth = 0.0, linealpha=1.0;
  double lineR=-1.0, lineG=-1.0, lineB=-1.0;

  double cornerwidth = 0.0;
  double corneralpha = 1.0;
  double cornerR=-1.0, cornerG=-1.0, cornerB=-1.0;

  int replace=1;

  strcpy(surfname,"defaultsurface");
  strcpy(curvetype,"polyline");

  i=1;
  while (i<argc) {
    if (streq(argv[i],"boxalpha") && ((i+1)<argc)) {
      i++; boxalpha=atof(argv[i]);
    } else if (streq(argv[i],"boxcolor") && ((i+3)<argc)) {
      i++; boxR=atof(argv[i]);
      i++; boxG=atof(argv[i]);
      i++; boxB=atof(argv[i]);

    } else if (streq(argv[i],"linewidth") && ((i+1)<argc)) {
      i++; linewidth=atof(argv[i]);
    } else if (streq(argv[i],"linealpha") && ((i+1)<argc)) {
      i++; linealpha=atof(argv[i]);
    } else if (streq(argv[i],"linecolor") && ((i+3)<argc)) {
      i++; lineR=atof(argv[i]);
      i++; lineG=atof(argv[i]);
      i++; lineB=atof(argv[i]);
    } else if (streq(argv[i],"linetype") && ((i+1)<argc)) {
      i++; strncpy(curvetype,argv[i],sizeof(curvetype));

    } else if (streq(argv[i],"cornerwidth") && ((i+1)<argc)) {
      i++; cornerwidth=atof(argv[i]);
    } else if (streq(argv[i],"corneralpha") && ((i+1)<argc)) {
      i++; corneralpha=atof(argv[i]);
    } else if (streq(argv[i],"cornercolor") && ((i+3)<argc)) {
      i++; cornerR=atof(argv[i]);
      i++; cornerG=atof(argv[i]);
      i++; cornerB=atof(argv[i]);

    } else if (streq(argv[i],"surface") && ((i+1)<argc)) {
      i++; strncpy(surfname,argv[i],sizeof(surfname));

    } else if (streq(argv[i],"replace") && ((i+1)<argc)) {
      i++; replace=atoi(argv[i]);

    } else {
      fprintf(stderr,"Unknown option: %s\n",argv[i]);
    }
    i++;
  }

  /* tell the renderer which commands to filter */

  printf("Procedural\n");
  fflush(stdout);

  /* process commands */

  while (fgets(buf,BUFSIZE,stdin) != NULL) {

    if (strstr(buf,"options")!=0) {  // assume Massive options proc
       printf(buf);
    } else {

    if (replace==0) printf(buf);

    ntk=Tokenize(buf,MAXTOKENS,tk);

    if (ntk>3) {
       bdp = tk[3];
       ntk= Tokenize(bdp,MAXTOKENS,tk);
       if (ntk==6) {
          printf("AttributeBegin\n");
          printf("Surface \"%s\"\n", surfname);

          if ((linewidth>0.0)&&(linealpha>0.0)) {

            printf("AttributeBegin\n");
              if (lineR>=0.0) printf("Color %g %g %g\n", lineR,lineG,lineB);
              printf("Opacity %g %g %g\n",linealpha,linealpha,linealpha);
              printf("Attribute \"curve\" \"type\" \"%s\"\n", curvetype);
              printf("Curves \"linear\" [2 2 2 2  2 2 2 2  2 2 2 2] \"nonperiodic\" \"constantwidth\" [%g]\n",linewidth);
              printf(" \"P\" [");
              printf(" %s %s %s  %s %s %s  %s %s %s  %s %s %s ", tk[0],tk[2],tk[4], tk[1],tk[2],tk[4], tk[1],tk[2],tk[4], tk[1],tk[3],tk[4]);
              printf(" %s %s %s  %s %s %s  %s %s %s  %s %s %s ", tk[1],tk[3],tk[4], tk[0],tk[3],tk[4], tk[0],tk[3],tk[4], tk[0],tk[2],tk[4]);

              printf(" %s %s %s  %s %s %s  %s %s %s  %s %s %s ", tk[0],tk[2],tk[5], tk[1],tk[2],tk[5], tk[1],tk[2],tk[5], tk[1],tk[3],tk[5]);
              printf(" %s %s %s  %s %s %s  %s %s %s  %s %s %s ", tk[1],tk[3],tk[5], tk[0],tk[3],tk[5], tk[0],tk[3],tk[5], tk[0],tk[2],tk[5]);

              printf(" %s %s %s  %s %s %s  %s %s %s  %s %s %s", tk[0],tk[2],tk[4], tk[0],tk[2],tk[5], tk[1],tk[2],tk[4], tk[1],tk[2],tk[5]);
              printf(" %s %s %s  %s %s %s  %s %s %s  %s %s %s]\n", tk[0],tk[3],tk[4], tk[0],tk[3],tk[5], tk[1],tk[3],tk[4], tk[1],tk[3],tk[5]);
            printf("AttributeEnd\n");
          }

          if ((cornerwidth>0.0)&&(corneralpha>0.0)) {

            printf("AttributeBegin\n");
              if (cornerR>=0.0) printf("Color %g %g %g\n", cornerR,cornerG,cornerB);
              printf("Opacity %g %g %g\n",corneralpha,corneralpha,corneralpha);
              printf("Attribute \"point\" \"type\" \"sphere\"\n");
              printf("Points \"constantwidth\" [%g]\n", cornerwidth);
              printf(" \"P\" [%s %s %s  %s %s %s  %s %s %s  %s %s %s",tk[0],tk[2],tk[4], tk[1],tk[2],tk[4],  tk[0],tk[3],tk[4], tk[1],tk[3], tk[4]);
              printf(" %s %s %s  %s %s %s  %s %s %s  %s %s %s]\n",tk[0],tk[2],tk[5], tk[1],tk[2],tk[5],  tk[0],tk[3],tk[5], tk[1],tk[3], tk[5]);
            printf("AttributeEnd\n");
          }

          if (boxalpha>0.0) {
            printf("AttributeBegin\n");
              if (boxR>=0.0) printf("Color %g %g %g\n", boxR,boxG,boxB);
              printf("Opacity %g %g %g\n",boxalpha,boxalpha,boxalpha);

              printf("PointsPolygons [4 4 4 4 4 4] [0 2 3 1  1 3 7 5  5 7 6 4  4 6 2 0  2 6 7 3  4 0 1 5]");
              printf(" \"P\" [%s %s %s  %s %s %s  %s %s %s  %s %s %s",tk[0],tk[2],tk[4], tk[1],tk[2],tk[4],  tk[0],tk[3],tk[4], tk[1],tk[3], tk[4]);
              printf(" %s %s %s  %s %s %s  %s %s %s  %s %s %s]\n",tk[0],tk[2],tk[5], tk[1],tk[2],tk[5],  tk[0],tk[3],tk[5], tk[1],tk[3], tk[5]);
            printf("AttributeEnd\n");
          }

          printf("AttributeEnd\n");
       }
    }
    }

    /* always send this! */
    printf("\377");
    fflush(stdout);
  }
  return 0;
}
